#include "Keypad.h"
#include <stdlib.h>
#include <string.h>
#include "lcd.h"

char Keypad::GetKeyPressed()
{
    uint8_t r,c;

	KEYPAD_PORT |= 0XF0;
	PORTB |= (1 << PB0);

	for(r = 0; r < NUMROW; r++) {
		KEYPAD_DDR &= ~(0XFF);	
        DDRB &= ~(1 << PB0);
		
		if( r == 4)
			DDRB |= (1 << PB0);
		else
			KEYPAD_DDR |= (1 << r);
			
		// pentru a apuca sa se seteze bitii pe uP
		_delay_ms(1); 
		
        for(c = 0; c < NUMCOL; c++)
        {
			
			if(!(KEYPAD_PIN & (0X80>>c)))
			   return decodeKey((c + 1) * NUMCOL + r);
			else
				if(!(PINB & (1 << PB0)))
					return decodeKey(c * NUMCOL + r);
			
		}
		
	}
	// No key pressed
      return KEY_NULL;
}

char Keypad::decodeKey(uint8_t key)
{
	switch(key) {
		case 0:
			return KEY_F1;
		case 1:
			return KEY_F2;
		case 2:
			return KEY_DIEZ;
		case 3:
			return KEY_STAR;
		case 4:
			return '1';
		case 5:
			return '2';
		case 6:
			return '3';
		case 7:
			return KEY_UP;
		case 8:
			return '4';
		case 9:
			return '5';
		case 10:
			return '6';
		case 11:
			return KEY_DOWN;
		case 12:
			return '7';
		case 13:
			return '8';
		case 14:
			return '9';
		case 15:
			return KEY_ESC;
		case 16:
			return KEY_LEFT;
		case 17:
			return '0';
		case 18:
			return KEY_RIGHT;
		case 19:
			return KEY_ENTER;
		default:
			return KEY_NULL;
	}
}

void Keypad::displayKey(char key)
{ 
	if(key != KEY_NULL) {
		if(!this->inc) {
			LCD_putCharAt(this->pos_key, key);
			this->pos_key++;
			this->inc = 1;

			if(this->pos_key >= 0x10 && this->pos_key < 0x40)
				this->pos_key = 0x40;
			if(this->pos_key >= 0x50)
				this->pos_key = 0x00;
		}
	}
	else 
		this->inc = 0;
}


uint32_t Keypad::getCode()
{
	return this->code;
}

uint8_t Keypad::canReturn()
{
	return this->canReturnVar;
}

void Keypad::resetAll()
{
	this->inc = 0;
	this->nrs = 0;
	this->pos_key = 0;
	this->code = 0;
	this->canReturnVar = 0;
}
void Keypad::readCode()
{
	char key = this->GetKeyPressed();
	
	
	if(key != KEY_NULL) {
		if(!this->inc) {
			
			this->pos_key = 0x40 + this->nrs;
			if(key >= '0' && key <= '9' && this-> nrs < 5)
			{
				code = code * 10 + (key - '0');
				this->nrs++;
				LCD_putCharAt(this->pos_key, key);
			}
			else
				if(key == KEY_ENTER && this->code > 999)
					this->canReturnVar = 1;
				else if(key == KEY_LEFT && this->nrs > 0) {
						this->nrs--;
						LCD_putCharAt(0x40 + this->nrs, ' ');
						LCD_moveCursorLeft();
						if(code > 0)
							code = code / 10;
						char cod[9];
						itoa(code, cod, 10);
						
						if(code != 0)
							LCD_printAt(0x40, cod);
					}
					else
						this->canReturnVar = 0;
			this->inc = 1;
		}
	}
	else
		this->inc = 0;
}
